import json
import os
import time

import requests
from notetool.tool.secret import read_secret
from wechatpy.enterprise import WeChatClient


class HuoBiMessage:
    def __init__(self, name='huobi', webhook=None, agent_id=None, secret=None, company_id=None):
        self.name = name
        # 机器人webhook
        self.webhook = read_secret("wechat", "notechats", "huobi", name, "webhook", value=webhook)
        # 应用ID
        self.agent_id = read_secret("wechat", "notechats", "huobi", name, "agent_id", value=agent_id)
        # 企业ID
        self.company_id = read_secret("wechat", "notechats", "huobi", name, "company_id", value=company_id)
        # 应用Secret
        self.secret = read_secret("wechat", "notechats", "huobi", name, "secret", value=secret)
        self.client = WeChatClient(corp_id=self.company_id, secret=self.secret)

    def send_msg(self, msg):
        self.check_token()
        data = {
            "msgtype": "text",
            "text": {
                "content": msg
            }
        }
        self.client.message.send(agent_id=self.agent_id, user_ids=['NiuLiangTao', 'GuoYe'], party_ids=['3'], msg=data)

        self.send_to_qywx(msg)

    def send_to_qywx(self, msg):
        headers = {'Content-Type': 'application/json'}
        data = {
            "msgtype": "text",
            "text": {
                "content": msg
            }
        }

        data = json.dumps(data)
        r = requests.post(url=self.webhook, headers=headers, data=data)

    def check_token(self):
        # 通行密钥
        access_token = None
        access_token = read_secret("wechat", "notechats", "huobi", self.name, "access_token")
        if not access_token:
            response = self.client.fetch_access_token()
            access_token = read_secret("wechat", "notechats", "huobi", self.name, "access_token",
                                       value=response['access_token'], expire_time=response['expires_in'])
            print(response)

        self.client.session.set(self.client.access_token_key, access_token)
        return access_token



from sqlalchemy import (BIGINT, Column, Float, String, Table, and_, asc, desc,
                        func, or_, select)

from .base import BaseTable, engine, meta


class KlineData(BaseTable):
    def __init__(self, table_name="kline_data_1min", source=None, *args, **kwargs):
        if source is not None:
            table_name = f"{source.lower()}_{table_name}"

        super(KlineData, self).__init__(table_name=table_name, *args, **kwargs)
        self.table = Table(self.table_name, meta,
                           Column('symbol', String(30), comment='symbol', primary_key=True),
                           Column('timestamp', BIGINT, comment='timestamp', primary_key=True),
                           Column('open', Float, comment='open', default=0),
                           Column('close', Float, comment='close', default=0),
                           Column('low', Float, comment='low', default=0),
                           Column('high', Float, comment='high', default=0),
                           Column('vol', Float, comment='vol', default=0),
                           extend_existing=True,
                           autoload=True
                           )
        self.create()

    def select_symbol_maxmin(self, symbol):
        self.create()
        s = select(self.table.columns.symbol,
                   func.max(self.table.columns.timestamp).label('max_time'),
                   func.min(self.table.columns.timestamp).label('min_time')
                   ).where(self.table.columns.symbol == symbol).group_by(self.table.columns.symbol)
        data = [line for line in engine.execute(s)]
        if len(data) == 1:
            return data[0][1], data[0][2]
        else:
            return 0, 0



import json
import logging
from typing import Dict, Optional

import requests
from notecoin.okex.common import exceptions
from notecoin.okex import utils
from notecoin.okex.types import Response

GET = "GET"
POST = "POST"
DELETE = "DELETE"
API_URL = 'https://www.okex.com'
SERVER_TIMESTAMP_URL = '/api/general/v5/time'


class BaseClient(object):

    def __init__(self, api_key, api_secret_key, passphrase, use_server_time=False, test=False, first=False,
                 api_url=None, flag='1', proxy=None):
        self.API_KEY = api_key
        self.API_SECRET_KEY = api_secret_key
        self.PASSPHRASE = passphrase
        self.use_server_time = use_server_time
        self.flag = flag
        self.first = first
        self.test = test
        self.api_url = api_url or API_URL
        self.proxy = proxy

    def _request(self, method, uri, params, cursor=False):
        if method == GET:
            uri = uri + utils.parse_params_to_str(params)

        url = self.api_url + uri

        # 获取本地时间
        timestamp = utils.get_timestamp()

        # sign & header
        if self.use_server_time:
            # 获取服务器时间
            timestamp = self._get_timestamp()

        if isinstance(params, str):
            body = params if method == POST else ""
        elif isinstance(params, dict):
            body = json.dumps(params) if method == POST else ""
        else:
            body = json.dumps(params) if method == POST else ""
        sign = utils.sign(utils.pre_hash(timestamp, method, uri, str(body)), self.API_SECRET_KEY)
        header = utils.get_header(self.API_KEY, sign, timestamp, self.PASSPHRASE)

        if self.test:
            header['x-simulated-trading'] = '1'
        if self.first:
            print("url:", url)
            self.first = False

        logging.debug("url: " + url)
        logging.debug("body: " + body)

        # send request
        response = None
        if method == GET:
            if self.proxy is None:
                response = requests.get(url, headers=header)
            else:
                response = requests.get(url, headers=header, proxies=self.proxy)
        elif method == POST:
            if self.proxy is None:
                response = requests.post(url, data=body, headers=header)
            else:
                response = requests.post(url, data=body, headers=header, proxies=self.proxy)
        elif method == DELETE:
            if self.proxy is None:
                response = requests.delete(url, headers=header)
            else:
                response = requests.delete(url, headers=header, proxies=self.proxy)

        # exception handle
        if not str(response.status_code).startswith('2'):
            raise exceptions.OkexAPIException(response)
        try:
            res_header = response.headers
            if cursor:
                r = dict()
                try:
                    r['before'] = res_header['OK-BEFORE']
                    r['after'] = res_header['OK-AFTER']
                except Exception as e:
                    logging.warning(f"error:{e}")
                    pass
                return Response(response.json()), r
            else:
                return Response(response.json())

        except ValueError:
            raise exceptions.OkexRequestException('Invalid Response: %s' % response.text)

    def _request_without_params(self, method, request_path):
        return self._request(method, request_path, {})

    def _request_with_params(self, method, request_path, params, cursor=False):
        param = {}
        for k, v in params.items():
            if v is not None:
                param[k] = v
        return self._request(method, request_path, param, cursor)

    def _get_timestamp(self):
        url = self.api_url + SERVER_TIMESTAMP_URL
        response = requests.get(url, proxies=self.proxy)
        if response.status_code == 200:
            # return response.json()['ts']
            return response.json()['iso']
        else:
            return ""

    def set_api_url(self, url: str):
        self.api_url = url

    def set_proxy(self, proxy: Optional[Dict]):
        self.proxy = proxy


class Client(BaseClient):
    pass



import json
import logging
import time
from datetime import datetime

import pandas as pd
from ccxt.base.exchange import Exchange
from notecoin.base.database import KlineData
from tqdm import tqdm

logger = logging.getLogger()
unix_month = 2678400000


class LoadDataKline:
    def __init__(self, exchange: Exchange, *args, **kwargs):
        self.table = KlineData(source=exchange.name)
        self.exchange = exchange
        super(LoadDataKline, self).__init__(*args, **kwargs)

    def load_all(self, *args, **kwargs):
        self.exchange.load_markets()
        for sym in tqdm(self.exchange.symbols):
            if ':' not in sym:
                self.load(sym, *args, **kwargs)

    def _load(self, symbol, fetch_since, timeframe='1m', limit=100, try_times=3, *args, **kwargs):
        if try_times <= 0:
            return False, None

        try:
            result = self.exchange.fetch_ohlcv(symbol, timeframe, fetch_since, limit=limit)
            result = self.exchange.sort_by(result, 0)
            if len(result) == 0:
                return False, f'result is empty, fetch_since:{fetch_since}'

            df = pd.DataFrame(result, columns=['timestamp', 'open', 'close', 'low', 'high', 'vol'])
            df['symbol'] = symbol
            self.table.insert(json.loads(df.to_json(orient='records')))
            time.sleep(int(self.exchange.rateLimit / 1000))
            return True, result
        except Exception as e:
            return self._load(symbol, fetch_since, timeframe, limit=limit, try_times=try_times - 1, *args, **kwargs)

    def load(self, symbol, unix_start, unix_end, timeframe='1m', limit=100, *args, **kwargs):
        unix_max, unix_min = self.table.select_symbol_maxmin(symbol)
        unix_delta = limit * self.exchange.parse_timeframe(timeframe) * 1000

        unix_start = unix_start
        unix_end = unix_end
        unix_max = unix_max or unix_end
        unix_min = unix_min or unix_end
        #print(unix_start, unix_end, unix_max, unix_max)

        if unix_min >= unix_start:
            unix_temp = unix_min
            pbar = tqdm(range(unix_min, unix_start, -unix_delta), desc=symbol)
            for _ in pbar:
                status, result = self._load(symbol, unix_temp - unix_delta, timeframe=timeframe, limit=limit)
                if status is False:
                    break
                pbar.set_postfix({'time': datetime.fromtimestamp(result[0][0] // 1000)})
                unix_temp = result[0][0]

        if unix_max >= unix_end:
            unix_temp = unix_max
            pbar = tqdm(range(unix_max, unix_end, unix_delta), desc=symbol)
            for _ in pbar:
                status, result = self._load(symbol, unix_temp, timeframe=timeframe, limit=limit)
                if status is False:
                    break
                unix_temp = result[-1][0]



import os
from datetime import datetime

from notedrive.lanzou import LanZouDrive


class DataFileProperty:
    def __init__(self, exchange, data_type='kline', path='~/workspace/tmp', start_date=datetime.today(),
                 end_date=datetime.today(), freq='daily', timeframe='1m', file_format='%Y%m%d'):
        self.path = path
        self.freq = freq
        self.exchange = exchange
        self.data_type = data_type
        self.timeframe = timeframe
        self.start_date = start_date
        self.end_date = end_date
        self.file_format = file_format
        self.exchange_name = exchange.name.lower()
        self.drive = LanZouDrive()
        self.drive.ignore_limits()
        self.drive.login_by_cookie()

    @property
    def file_path_dir(self):
        path = f"{self.path}/notecoin/{self.exchange_name}/{self.data_type}-{self.freq}-{self.timeframe}"
        if not os.path.exists(path):
            os.makedirs(path)
        return path

    @property
    def filename_prefix(self):
        return f"{self.exchange_name}-{self.data_type}-{self.freq}-{self.timeframe}-{self.start_date.strftime(self.file_format)}"

    @property
    def file_path_csv(self):
        return f"{self.file_path_dir}/{self.filename_prefix}.csv"

    @property
    def file_path_tar(self):
        return f"{self.file_path_dir}/{self.filename_prefix}.tar.gz"

    @property
    def file_path_tar_gz(self):
        return f"{self.file_path_dir}/{self.filename_prefix}.tar.gz"

    @property
    def file_path_tar_xz(self):
        return f"{self.file_path_dir}/{self.filename_prefix}.tar.xz"

    def arcname(self, file):
        path = self.file_path_dir.replace(f"{self.path}/", "")
        return os.path.join(path, os.path.basename(file))

    def sync(self):
        self.drive.sync_files(self.path, '5679873', only_directory=False)



import time


def get_current_timestamp():
    return int(round(time.time() * 1000))


def convert_cst_in_second_to_utc(time_in_second):
    time_in_second = int(time_in_second)
    if time_in_second > 946656000 and time_in_second < 946656000000:
        return (time_in_second - 8 * 60 * 60) * 1000
    else:
        return 0


def convert_cst_in_millisecond_to_utc(time_in_ms):
    time_in_ms = int(time_in_ms)
    if time_in_ms > 946656000000:
        return time_in_ms - 8 * 60 * 60 * 1000
    else:
        return 0

